@@ -19,8 +19,10 @@ from paramiko.common import linefeed_byte_value, crlf, cr_byte, linefeed_byte, \
     cr_byte_value
 from paramiko.py3compat import BytesIO, PY2, u, b, bytes_types
 
+from paramiko.util import ClosingContextManager
 
-class BufferedFile (object):
+
+class BufferedFile (ClosingContextManager):
     """
     Reusable base class to implement Python-style file buffering around a
     simpler stream.
@@ -204,7 +206,6 @@ class BufferedFile (object):
         if not (self._flags & self.FLAG_READ):
             raise IOError('File not open for reading')
         line = self._rbuffer
-        truncated = False
         while True:
             if self._at_trailing_cr and (self._flags & self.FLAG_UNIVERSAL_NEWLINE) and (len(line) > 0):
                 # edge case: the newline may be '\r\n' and we may have read
@@ -219,11 +220,11 @@ class BufferedFile (object):
             # enough.
             if (size is not None) and (size >= 0):
                 if len(line) >= size:
-                    # truncate line
+                    # truncate line and return
                     self._rbuffer = line[size:]
                     line = line[:size]
-                    truncated = True
-                    break
+                    self._pos += len(line)
+                    return line if self._flags & self.FLAG_BINARY else u(line)
                 n = size - len(line)
             else:
                 n = self._bufsize
@@ -245,17 +246,10 @@ class BufferedFile (object):
             rpos = line.find(cr_byte)
             if (rpos >= 0) and (rpos < pos or pos < 0):
                 pos = rpos
-        if pos == -1:
-            # we couldn't find a newline in the truncated string, return it
-            self._pos += len(line)
-            return line if self._flags & self.FLAG_BINARY else u(line)
         xpos = pos + 1
         if (line[pos] == cr_byte_value) and (xpos < len(line)) and (line[xpos] == linefeed_byte_value):
             xpos += 1
-        # if the string was truncated, _rbuffer needs to have the string after 
-        # the newline character plus the truncated part of the line we stored
-        # earlier in _rbuffer
-        self._rbuffer = line[xpos:] + self._rbuffer if truncated else line[xpos:]
+        self._rbuffer = line[xpos:]
         lf = line[pos:xpos]
         line = line[:pos] + linefeed_byte
         if (len(self._rbuffer) == 0) and (lf == cr_byte):
