@@ -23,6 +23,7 @@ Useful functions used by the rest of paramiko.
 from __future__ import generators
 
 import array
+from binascii import hexlify, unhexlify
 import errno
 import sys
 import struct
@@ -105,14 +106,21 @@ def format_binary_line(data):
     return '%-50s %s' % (left, right)
 
 
+def hexify(s):
+    return hexlify(s).upper()
+
+
+def unhexify(s):
+    return unhexlify(s)
+
+
 def safe_string(s):
-    out = b('')
+    out = ''
     for c in s:
-        i = byte_ord(c)
-        if 32 <= i <= 127:
-            out += byte_chr(i)
+        if (byte_ord(c) >= 32) and (byte_ord(c) <= 127):
+            out += c
         else:
-            out += b('%%%02X' % i)
+            out += '%%%02X' % byte_ord(c)
     return out
 
 
@@ -312,3 +320,15 @@ def constant_time_bytes_eq(a, b):
     for i in (xrange if PY2 else range)(len(a)):
         res |= byte_ord(a[i]) ^ byte_ord(b[i])
     return res == 0
+
+
+class ClosingContextManager(object):
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type, value, traceback):
+        self.close()
+
+
+def clamp_value(minimum, val, maximum):
+    return max(minimum, min(val, maximum))
