@@ -7,7 +7,7 @@
 # Software Foundation; either version 2.1 of the License, or (at your option)
 # any later version.
 #
-# Paramiko is distrubuted in the hope that it will be useful, but WITHOUT ANY
+# Paramiko is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 # A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 # details.
@@ -20,11 +20,11 @@
 Some unit tests for HostKeys.
 """
 
-import base64
 from binascii import hexlify
 import os
 import unittest
 import paramiko
+from paramiko.py3compat import decodebytes
 
 
 test_hosts_file = """\
@@ -36,12 +36,12 @@ BGQ3GQ/Fc7SX6gkpXkwcZryoi4kNFhHu5LvHcZPdxXV1D+uTMfGS1eyd2Yz/DoNWXNAl8TI0cAsW\
 5ymME3bQ4J/k1IKxCtz/bAlAqFgKoc+EolMziDYqWIATtW0rYTJvzGAzTmMj80/QpsFH+Pc2M=
 """
 
-keyblob = """\
+keyblob = b"""\
 AAAAB3NzaC1yc2EAAAABIwAAAIEA8bP1ZA7DCZDB9J0s50l31MBGQ3GQ/Fc7SX6gkpXkwcZryoi4k\
 NFhHu5LvHcZPdxXV1D+uTMfGS1eyd2Yz/DoNWXNAl8TI0cAsW5ymME3bQ4J/k1IKxCtz/bAlAqFgK\
 oc+EolMziDYqWIATtW0rYTJvzGAzTmMj80/QpsFH+Pc2M="""
 
-keyblob_dss = """\
+keyblob_dss = b"""\
 AAAAB3NzaC1kc3MAAACBAOeBpgNnfRzr/twmAQRu2XwWAp3CFtrVnug6s6fgwj/oLjYbVtjAy6pl/\
 h0EKCWx2rf1IetyNsTxWrniA9I6HeDj65X1FyDkg6g8tvCnaNB8Xp/UUhuzHuGsMIipRxBxw9LF60\
 8EqZcj1E3ytktoW5B5OcjrkEoz3xG7C+rpIjYvAAAAFQDwz4UnmsGiSNu5iqjn3uTzwUpshwAAAIE\
@@ -55,51 +55,50 @@ Ngw3qIch/WgRmMHy4kBq1SsXMjQCte1So6HBMvBPIW5SiMTmjCfZZiw4AYHK+B/JaOwaG9yRg2Ejg\
 class HostKeysTest (unittest.TestCase):
 
     def setUp(self):
-        f = open('hostfile.temp', 'w')
-        f.write(test_hosts_file)
-        f.close()
+        with open('hostfile.temp', 'w') as f:
+            f.write(test_hosts_file)
 
     def tearDown(self):
         os.unlink('hostfile.temp')
 
     def test_1_load(self):
         hostdict = paramiko.HostKeys('hostfile.temp')
-        self.assertEquals(2, len(hostdict))
-        self.assertEquals(1, len(hostdict.values()[0]))
-        self.assertEquals(1, len(hostdict.values()[1]))
+        self.assertEqual(2, len(hostdict))
+        self.assertEqual(1, len(list(hostdict.values())[0]))
+        self.assertEqual(1, len(list(hostdict.values())[1]))
         fp = hexlify(hostdict['secure.example.com']['ssh-rsa'].get_fingerprint()).upper()
-        self.assertEquals('E6684DB30E109B67B70FF1DC5C7F1363', fp)
+        self.assertEqual(b'E6684DB30E109B67B70FF1DC5C7F1363', fp)
 
     def test_2_add(self):
         hostdict = paramiko.HostKeys('hostfile.temp')
         hh = '|1|BMsIC6cUIP2zBuXR3t2LRcJYjzM=|hpkJMysjTk/+zzUUzxQEa2ieq6c='
-        key = paramiko.RSAKey(data=base64.decodestring(keyblob))
+        key = paramiko.RSAKey(data=decodebytes(keyblob))
         hostdict.add(hh, 'ssh-rsa', key)
-        self.assertEquals(3, len(hostdict))
+        self.assertEqual(3, len(list(hostdict)))
         x = hostdict['foo.example.com']
         fp = hexlify(x['ssh-rsa'].get_fingerprint()).upper()
-        self.assertEquals('7EC91BB336CB6D810B124B1353C32396', fp)
-        self.assert_(hostdict.check('foo.example.com', key))
+        self.assertEqual(b'7EC91BB336CB6D810B124B1353C32396', fp)
+        self.assertTrue(hostdict.check('foo.example.com', key))
 
     def test_3_dict(self):
         hostdict = paramiko.HostKeys('hostfile.temp')
-        self.assert_('secure.example.com' in hostdict)
-        self.assert_('not.example.com' not in hostdict)
-        self.assert_(hostdict.has_key('secure.example.com'))
-        self.assert_(not hostdict.has_key('not.example.com'))
+        self.assertTrue('secure.example.com' in hostdict)
+        self.assertTrue('not.example.com' not in hostdict)
+        self.assertTrue('secure.example.com' in hostdict)
+        self.assertTrue('not.example.com' not in hostdict)
         x = hostdict.get('secure.example.com', None)
-        self.assert_(x is not None)
+        self.assertTrue(x is not None)
         fp = hexlify(x['ssh-rsa'].get_fingerprint()).upper()
-        self.assertEquals('E6684DB30E109B67B70FF1DC5C7F1363', fp)
+        self.assertEqual(b'E6684DB30E109B67B70FF1DC5C7F1363', fp)
         i = 0
         for key in hostdict:
             i += 1
-        self.assertEquals(2, i)
+        self.assertEqual(2, i)
         
     def test_4_dict_set(self):
         hostdict = paramiko.HostKeys('hostfile.temp')
-        key = paramiko.RSAKey(data=base64.decodestring(keyblob))
-        key_dss = paramiko.DSSKey(data=base64.decodestring(keyblob_dss))
+        key = paramiko.RSAKey(data=decodebytes(keyblob))
+        key_dss = paramiko.DSSKey(data=decodebytes(keyblob_dss))
         hostdict['secure.example.com'] = {
             'ssh-rsa': key,
             'ssh-dss': key_dss
@@ -107,11 +106,11 @@ class HostKeysTest (unittest.TestCase):
         hostdict['fake.example.com'] = {}
         hostdict['fake.example.com']['ssh-rsa'] = key
         
-        self.assertEquals(3, len(hostdict))
-        self.assertEquals(2, len(hostdict.values()[0]))
-        self.assertEquals(1, len(hostdict.values()[1]))
-        self.assertEquals(1, len(hostdict.values()[2]))
+        self.assertEqual(3, len(hostdict))
+        self.assertEqual(2, len(list(hostdict.values())[0]))
+        self.assertEqual(1, len(list(hostdict.values())[1]))
+        self.assertEqual(1, len(list(hostdict.values())[2]))
         fp = hexlify(hostdict['secure.example.com']['ssh-rsa'].get_fingerprint()).upper()
-        self.assertEquals('7EC91BB336CB6D810B124B1353C32396', fp)
+        self.assertEqual(b'7EC91BB336CB6D810B124B1353C32396', fp)
         fp = hexlify(hostdict['secure.example.com']['ssh-dss'].get_fingerprint()).upper()
-        self.assertEquals('4478F0B9A23CC5182009FF755BC1D26C', fp)
+        self.assertEqual(b'4478F0B9A23CC5182009FF755BC1D26C', fp)
